@@ -1,3 +1,7 @@
+# SPDX-FileCopyrightText: 2016 Sebastian Wagner
+#
+# SPDX-License-Identifier: AGPL-3.0-or-later
+
 # -*- coding: utf-8 -*-
 import argparse
 import datetime
@@ -21,9 +25,10 @@ from collections import OrderedDict
 
 import pkg_resources
 from termstyle import green
+from ruamel.yaml import YAML
 
-from intelmq import (BOTS_FILE, DEFAULT_LOGGING_LEVEL, DEFAULTS_CONF_FILE,  # noqa: F401
-                     HARMONIZATION_CONF_FILE, PIPELINE_CONF_FILE,
+from intelmq import (DEFAULT_LOGGING_LEVEL,  # noqa: F401
+                     HARMONIZATION_CONF_FILE,
                      RUNTIME_CONF_FILE, VAR_RUN_PATH, STATE_FILE_PATH,
                      DEFAULT_LOGGING_PATH, __version_info__,
                      CONFIG_DIR, ROOT_DIR)
@@ -34,6 +39,8 @@ from intelmq.lib.pipeline import PipelineFactory
 import intelmq.lib.upgrades as upgrades
 from typing import Union, Iterable
 
+yaml = YAML(typ="safe", pure=True)
+
 try:
     import psutil
 except ImportError:
@@ -695,6 +702,10 @@ class IntelMQController():
         QUIET = quiet
         self.parameters = Parameters()
 
+        # set default values for the logging handler
+        self.parameters.logging_handler = 'file'
+        self.parameters.logging_path = '/opt/intelmq/var/log'
+
         # Try to get logging_level from defaults configuration, else use default (defined above)
         defaults_loading_exc = None
         try:
@@ -703,7 +714,7 @@ class IntelMQController():
             defaults_loading_exc = exc
             logging_level_stream = 'DEBUG'
         else:
-            self.logging_level = self.parameters.logging_level.upper()
+            self.logging_level = getattr(self, 'logging_level', DEFAULT_LOGGING_LEVEL).upper()
         # make sure that logging_level_stream is always at least INFO or more verbose
         # otherwise the output on stdout/stderr is less than the user expects
         logging_level_stream = self.logging_level if self.logging_level == 'DEBUG' else 'INFO'
@@ -813,13 +824,6 @@ Get some debugging output on the settings and the environment (to be extended):
     intelmqctl debug --get-environment-variables
 '''
 
-        # stolen functions from the bot file
-        # this will not work with various instances of REDIS
-        try:
-            self.pipeline_configuration = utils.load_configuration(PIPELINE_CONF_FILE)
-        except ValueError as exc:  # pragma: no cover
-            self.abort('Error loading %r: %s' % (PIPELINE_CONF_FILE, exc))
-
         try:
             self.runtime_configuration = utils.load_configuration(RUNTIME_CONF_FILE)
         except ValueError as exc:  # pragma: no cover
@@ -865,6 +869,8 @@ Get some debugging output on the settings and the environment (to be extended):
                                      help='Only show the total '
                                           'number of messages in queues. '
                                           'Only valid for listing queues.')
+            parser_list.add_argument('--configured', '-c', action='store_true',
+                                     help='Only show configured bots')
             parser_list.set_defaults(func=self.list)
 
             parser_clear = subparsers.add_parser('clear', help='Clear a queue')
@@ -872,7 +878,7 @@ Get some debugging output on the settings and the environment (to be extended):
             parser_clear.set_defaults(func=self.clear_queue)
 
             parser_log = subparsers.add_parser('log', help='Get last log lines of a bot')
-            parser_log.add_argument('bot_id', help='bot id')
+            parser_log.add_argument('bot_id', help='bot id', choices=self._configured_bots_list())
             parser_log.add_argument('number_of_lines', help='number of lines',
                                     default=10, type=int, nargs='?')
             parser_log.add_argument('log_level', help='logging level',
@@ -881,7 +887,7 @@ Get some debugging output on the settings and the environment (to be extended):
 
             parser_run = subparsers.add_parser('run', help='Run a bot interactively')
             parser_run.add_argument('bot_id',
-                                    choices=self.runtime_configuration.keys())
+                                    choices=self._configured_bots_list())
             parser_run.add_argument('--loglevel', '-l',
                                     nargs='?', default=None,
                                     choices=LOG_LEVEL.keys())
@@ -931,47 +937,47 @@ Get some debugging output on the settings and the environment (to be extended):
 
             parser_start = subparsers.add_parser('start', help='Start a bot or botnet')
             parser_start.add_argument('bot_id', nargs='?',
-                                      choices=self.runtime_configuration.keys())
+                                      choices=self._configured_bots_list())
             parser_start.add_argument('--group', help='Start a group of bots',
                                       choices=BOT_GROUP.keys())
             parser_start.set_defaults(func=self.bot_start)
 
             parser_stop = subparsers.add_parser('stop', help='Stop a bot or botnet')
             parser_stop.add_argument('bot_id', nargs='?',
-                                     choices=self.runtime_configuration.keys())
+                                     choices=self._configured_bots_list())
             parser_stop.add_argument('--group', help='Stop a group of bots',
                                      choices=BOT_GROUP.keys())
             parser_stop.set_defaults(func=self.bot_stop)
 
             parser_restart = subparsers.add_parser('restart', help='Restart a bot or botnet')
             parser_restart.add_argument('bot_id', nargs='?',
-                                        choices=self.runtime_configuration.keys())
+                                        choices=self._configured_bots_list())
             parser_restart.add_argument('--group', help='Restart a group of bots',
                                         choices=BOT_GROUP.keys())
             parser_restart.set_defaults(func=self.bot_restart)
 
             parser_reload = subparsers.add_parser('reload', help='Reload a bot or botnet')
             parser_reload.add_argument('bot_id', nargs='?',
-                                       choices=self.runtime_configuration.keys())
+                                       choices=self._configured_bots_list())
             parser_reload.add_argument('--group', help='Reload a group of bots',
                                        choices=BOT_GROUP.keys())
             parser_reload.set_defaults(func=self.bot_reload)
 
             parser_status = subparsers.add_parser('status', help='Status of a bot or botnet')
             parser_status.add_argument('bot_id', nargs='?',
-                                       choices=self.runtime_configuration.keys())
+                                       choices=self._configured_bots_list())
             parser_status.add_argument('--group', help='Get status of a group of bots',
                                        choices=BOT_GROUP.keys())
             parser_status.set_defaults(func=self.bot_status)
 
             parser_status = subparsers.add_parser('enable', help='Enable a bot')
             parser_status.add_argument('bot_id',
-                                       choices=self.runtime_configuration.keys())
+                                       choices=self._configured_bots_list())
             parser_status.set_defaults(func=self.bot_enable)
 
             parser_status = subparsers.add_parser('disable', help='Disable a bot')
             parser_status.add_argument('bot_id',
-                                       choices=self.runtime_configuration.keys())
+                                       choices=self._configured_bots_list())
             parser_status.set_defaults(func=self.bot_disable)
 
             parser_upgrade_conf = subparsers.add_parser('upgrade-config',
@@ -1008,13 +1014,7 @@ Get some debugging output on the settings and the environment (to be extended):
             self.parser = parser
 
     def load_defaults_configuration(self, silent=False):
-        # Load defaults configuration
-        try:
-            config = utils.load_configuration(DEFAULTS_CONF_FILE)
-        except ValueError as exc:  # pragma: no cover
-            if not silent:
-                self.abort('Error loading %r: %s' % (DEFAULTS_CONF_FILE, exc))
-        for option, value in config.items():
+        for option, value in utils.get_global_settings().items():
             setattr(self.parameters, option, value)
 
         # TODO: Rewrite variables with env. variables ( CURRENT IMPLEMENTATION NOT FINAL )
@@ -1130,14 +1130,21 @@ Get some debugging output on the settings and the environment (to be extended):
     def _is_enabled(self, bot_id):
         return self.runtime_configuration[bot_id].get('enabled', True)
 
+    def _configured_bots_list(self, group=None):
+        if group is not None:
+            bots = sorted(k_v[0] for k_v in filter(lambda x: x[1]["group"] == BOT_GROUP[group], self.runtime_configuration.items()))
+        else:
+            bots = sorted(self.runtime_configuration.keys())
+        if 'global' in bots:
+            bots.remove('global')
+        return bots
+
     def botnet_start(self, group=None):
         botnet_status = {}
         log_botnet_message('starting', group)
 
-        if group:
-            bots = sorted(k_v[0] for k_v in filter(lambda x: x[1]["group"] == BOT_GROUP[group], self.runtime_configuration.items()))
-        else:
-            bots = sorted(self.runtime_configuration.keys())
+        bots = self._configured_bots_list(group=group)
+
         for bot_id in bots:
             if self.runtime_configuration[bot_id].get('enabled', True):
                 self.bot_start(bot_id, getstatus=False)
@@ -1162,10 +1169,8 @@ Get some debugging output on the settings and the environment (to be extended):
         botnet_status = {}
         log_botnet_message('stopping', group)
 
-        if group:
-            bots = sorted(k_v[0] for k_v in filter(lambda x: x[1]["group"] == BOT_GROUP[group], self.runtime_configuration.items()))
-        else:
-            bots = sorted(self.runtime_configuration.keys())
+        bots = self._configured_bots_list(group=group)
+
         for bot_id in bots:
             self.bot_stop(bot_id, getstatus=False)
 
@@ -1183,10 +1188,7 @@ Get some debugging output on the settings and the environment (to be extended):
         botnet_status = {}
         log_botnet_message('reloading', group)
 
-        if group:
-            bots = sorted(k_v[0] for k_v in filter(lambda x: x[1]["group"] == BOT_GROUP[group], self.runtime_configuration.items()))
-        else:
-            bots = sorted(self.runtime_configuration.keys())
+        bots = sorted(self._configured_bots_list(group=group))
         for bot_id in bots:
             self.bot_reload(bot_id, getstatus=False)
 
@@ -1210,21 +1212,18 @@ Get some debugging output on the settings and the environment (to be extended):
     def botnet_status(self, group=None):
         retval = 0
         botnet_status = {}
-        if group:
-            bots = sorted(k_v[0] for k_v in filter(lambda x: x[1]["group"] == BOT_GROUP[group], self.runtime_configuration.items()))
-        else:
-            bots = sorted(self.runtime_configuration.keys())
+        bots = sorted(self._configured_bots_list(group=group))
         for bot_id in bots:
             botnet_status[bot_id] = self.bot_status(bot_id)[1]
             if botnet_status[bot_id] not in ['running', 'disabled']:
                 retval = 1
         return retval, botnet_status
 
-    def list(self, kind=None, non_zero=False, count=False):
+    def list(self, kind=None, non_zero=False, count=False, configured=False):
         if kind == 'queues':
             return self.list_queues(non_zero=non_zero, count=count)
         elif kind == 'bots':
-            return self.list_bots(non_zero=non_zero)
+            return self.list_bots(non_zero=non_zero, configured=configured)
         elif kind == 'queues-and-status':
             q = self.list_queues()
             b = self.botnet_status()
@@ -1243,25 +1242,47 @@ Get some debugging output on the settings and the environment (to be extended):
             self.abort('Can\'t update runtime configuration: Permission denied.')
         return True
 
-    def list_bots(self, non_zero=False):
+    def list_bots(self, non_zero=False, configured=False):
         """
-        Lists all configured bots from runtime.conf with bot id and
-        description.
+        Lists all (configured) bots from runtime.conf or generated on demand
+        with bot id/module and description and parameters.
 
         If description is not set, None is used instead.
         """
-        if RETURN_TYPE == 'text':
-            for bot_id in sorted(self.runtime_configuration.keys(), key=str.lower):
-                if non_zero and not self.runtime_configuration[bot_id].get('enabled'):
-                    continue
-                if QUIET:
-                    print(bot_id)
-                else:
-                    print("Bot ID: {}\nDescription: {}"
-                          "".format(bot_id, self.runtime_configuration[bot_id].get('description')))
-        return 0, [{'id': bot_id,
-                    'description': self.runtime_configuration[bot_id].get('description')}
-                   for bot_id in sorted(self.runtime_configuration.keys())]
+        if configured:
+            if RETURN_TYPE == 'text':
+                for bot_id in self._configured_bots_list():
+                    if non_zero and not self.runtime_configuration[bot_id].get('enabled'):
+                        continue
+                    if QUIET:
+                        print(bot_id)
+                    else:
+                        print("Bot ID: {}\nDescription: {}"
+                              "".format(bot_id, self.runtime_configuration[bot_id].get('description')))
+            return 0, [{'id': bot_id,
+                        'description': self.runtime_configuration[bot_id].get('description')}
+                       for bot_id in sorted(self._configured_bots_list())]
+        else:
+            bots = utils.list_all_bots()
+            if RETURN_TYPE == 'text':
+                for bot_type in bots:
+                    print(f"\n======== {bot_type} ========\n")
+                    for bot in bots[bot_type]:
+                        print("Bot ID: {}\nDescription: {}"
+                              "".format(bot, bots[bot_type][bot].get('description')))
+            return 0, bots
+
+    def _pipeline_configuration(self):
+        pipeline_configuration = {}
+        for botid, botconfig in self.runtime_configuration.items():
+            if botid != 'global':
+                pipeline_configuration[botid] = {"source_queue": f"{botid}-queue", "destination_queues": []}
+                if 'parameters' in botconfig:
+                    if 'source_queue' in botconfig['parameters']:
+                        pipeline_configuration[botid]['source_queue'] = botconfig['parameters']['source_queue']
+                    if 'destination_queues' in botconfig['parameters']:
+                        pipeline_configuration[botid]['destination_queues'] = botconfig['parameters']['destination_queues']
+        return pipeline_configuration
 
     def get_queues(self, with_internal_queues=False):
         """
@@ -1274,21 +1295,21 @@ Get some debugging output on the settings and the environment (to be extended):
         destination_queues = set()
         internal_queues = set()
 
-        for botid, value in self.pipeline_configuration.items():
-            if 'source-queue' in value:
-                source_queues.add(value['source-queue'])
+        for botid, value in self._pipeline_configuration().items():
+            if 'source_queue' in value:
+                source_queues.add(value['source_queue'])
                 if with_internal_queues:
-                    internal_queues.add(value['source-queue'] + '-internal')
-            if 'destination-queues' in value:
+                    internal_queues.add(value['source_queue'] + '-internal')
+            if 'destination_queues' in value:
                 # flattens ["one", "two"] → {"one", "two"}, {"_default": "one", "other": ["two", "three"]} → {"one", "two", "three"}
-                destination_queues.update(utils.flatten_queues(value['destination-queues']))
+                destination_queues.update(utils.flatten_queues(value['destination_queues']))
 
         all_queues = source_queues.union(destination_queues).union(internal_queues)
 
         return source_queues, destination_queues, internal_queues, all_queues
 
     def list_queues(self, non_zero=False, count=False):
-        pipeline = PipelineFactory.create(self.parameters, logger=self.logger)
+        pipeline = PipelineFactory.create(logger=self.logger, pipeline_args=self.parameters.__dict__)
         pipeline.set_queues(None, "source")
         pipeline.connect()
         source_queues, destination_queues, internal_queues,\
@@ -1307,18 +1328,21 @@ Get some debugging output on the settings and the environment (to be extended):
         if count:
             return_dict = {'total-messages': sum(counters.values())}
         else:
-            for bot_id, info in self.pipeline_configuration.items():
+            for bot_id, info in self._pipeline_configuration().items():
                 return_dict[bot_id] = {}
 
-                if 'source-queue' in info:
+                # Do not report source queues for collectors
+                if 'source_queue' in info and not (
+                        self.runtime_configuration[bot_id].get('group', None) == 'Collector' or
+                        self.runtime_configuration[bot_id].get('groupname', None) == 'collectors'):
                     return_dict[bot_id]['source_queue'] = (
-                        info['source-queue'], counters[info['source-queue']])
+                        info['source_queue'], counters[info['source_queue']])
                     if pipeline.has_internal_queues:
-                        return_dict[bot_id]['internal_queue'] = counters[info['source-queue'] + '-internal']
+                        return_dict[bot_id]['internal_queue'] = counters[info['source_queue'] + '-internal']
 
-                if 'destination-queues' in info:
+                if 'destination_queues' in info:
                     return_dict[bot_id]['destination_queues'] = []
-                    for dest_queue in utils.flatten_queues(info['destination-queues']):
+                    for dest_queue in utils.flatten_queues(info['destination_queues']):
                         return_dict[bot_id]['destination_queues'].append((dest_queue, counters[dest_queue]))
 
         return 0, return_dict
@@ -1329,20 +1353,20 @@ Get some debugging output on the settings and the environment (to be extended):
 
         First checks if the queue does exist in the pipeline configuration.
         """
-        pipeline = PipelineFactory.create(self.parameters, logger=self.logger)
+        pipeline = PipelineFactory.create(logger=self.logger, pipeline_args=self.parameters.__dict__)
         pipeline.set_queues(None, "source")
         pipeline.connect()
 
         if RETURN_TYPE == 'text':
             logger.info("Clearing queue %s.", queue)
         queues = set()
-        for key, value in self.pipeline_configuration.items():
-            if 'source-queue' in value:
-                queues.add(value['source-queue'])
+        for key, value in self._pipeline_configuration().items():
+            if 'source_queue' in value:
+                queues.add(value['source_queue'])
                 if pipeline.has_internal_queues:
-                    queues.add(value['source-queue'] + '-internal')
-            if 'destination-queues' in value:
-                queues.update(value['destination-queues'])
+                    queues.add(value['source_queue'] + '-internal')
+            if 'destination_queues' in value:
+                queues.update(value['destination_queues'])
 
         if queue not in queues:
             if RETURN_TYPE == 'text':
@@ -1420,17 +1444,20 @@ Get some debugging output on the settings and the environment (to be extended):
             check_logger = self.logger
 
         # loading files and syntax check
-        files = {DEFAULTS_CONF_FILE: None, PIPELINE_CONF_FILE: None,
-                 RUNTIME_CONF_FILE: None, BOTS_FILE: None,
-                 HARMONIZATION_CONF_FILE: None}
+        files = {RUNTIME_CONF_FILE: None, HARMONIZATION_CONF_FILE: None}
         check_logger.info('Reading configuration files.')
-        for filename in files:
-            try:
-                with open(filename) as file_handle:
-                    files[filename] = json.load(file_handle)
-            except (IOError, ValueError) as exc:  # pragma: no cover
-                check_logger.error('Could not load %r: %s.', filename, exc)
-                retval = 1
+        try:
+            with open(HARMONIZATION_CONF_FILE) as file_handle:
+                files[HARMONIZATION_CONF_FILE] = json.load(file_handle)
+        except (IOError, ValueError) as exc:  # pragma: no cover
+            check_logger.error('Could not load %r: %s.', HARMONIZATION_CONF_FILE, exc)
+            retval = 1
+        try:
+            with open(RUNTIME_CONF_FILE) as file_handle:
+                files[RUNTIME_CONF_FILE] = yaml.load(file_handle)
+        except (IOError, ValueError) as exc:
+            check_logger.error('Could not load %r: %s.', RUNTIME_CONF_FILE, exc)
+            retval = 1
         if retval:
             if RETURN_TYPE == 'json':
                 return 1, {'status': 'error', 'lines': list_handler.buffer}
@@ -1438,28 +1465,11 @@ Get some debugging output on the settings and the environment (to be extended):
                 self.logger.error('Fatal errors occurred.')
                 return 1, retval
 
-        check_logger.info('Checking defaults configuration.')
-        try:
-            with open(pkg_resources.resource_filename('intelmq', 'etc/defaults.conf')) as fh:
-                defaults = json.load(fh)
-        except FileNotFoundError:
-            pass
-        else:
-            keys = set(defaults.keys()) - set(files[DEFAULTS_CONF_FILE].keys())
-            if keys:
-                check_logger.error("Keys missing in your 'defaults.conf' file: %r", keys)
-
-        check_logger.info('Checking runtime configuration.')
-        http_proxy = files[DEFAULTS_CONF_FILE].get('http_proxy')
-        https_proxy = files[DEFAULTS_CONF_FILE].get('https_proxy')
-        # Either both are given or both are not given
-        if (not http_proxy or not https_proxy) and not (http_proxy == https_proxy):
-            check_logger.warning('Incomplete configuration: Both http and https proxies must be set.')
-            retval = 1
-
         check_logger.info('Checking runtime and pipeline configuration.')
         all_queues = set()
         for bot_id, bot_config in files[RUNTIME_CONF_FILE].items():
+            if bot_id == 'global':
+                continue
             # pipeline keys
             for field in ['description', 'group', 'module', 'name', 'enabled']:
                 if field not in bot_config:
@@ -1469,37 +1479,26 @@ Get some debugging output on the settings and the environment (to be extended):
                 message = "Bot %r has invalid `run_mode` %r. Must be 'continuous' or 'scheduled'."
                 check_logger.warning(message, bot_id, bot_config['run_mode'])
                 retval = 1
-            if bot_id not in files[PIPELINE_CONF_FILE] and bot_config.get('enabled', True):
-                check_logger.error('Misconfiguration: No pipeline configuration found for %r.', bot_id)
-                retval = 1
-            elif bot_id not in files[PIPELINE_CONF_FILE] and not bot_config.get('enabled', True):
-                check_logger.warning('Misconfiguration: No pipeline configuration found for %r.', bot_id)
-            elif bot_id in files[PIPELINE_CONF_FILE]:
-                if ('group' in bot_config and
-                        bot_config['group'] in ['Collector', 'Parser', 'Expert']):
-                    if ('destination-queues' not in files[PIPELINE_CONF_FILE][bot_id] or
-                            (isinstance(files[PIPELINE_CONF_FILE][bot_id]['destination-queues'], list) and
-                             len(files[PIPELINE_CONF_FILE][bot_id]['destination-queues']) < 1) or
-                            (isinstance(files[PIPELINE_CONF_FILE][bot_id]['destination-queues'], dict) and
-                             '_default' not in files[PIPELINE_CONF_FILE][bot_id]['destination-queues'])):
-                        check_logger.error('Misconfiguration: No (default) destination queue for %r.', bot_id)
-                        retval = 1
-                    else:
-                        all_queues = all_queues.union(files[PIPELINE_CONF_FILE][bot_id]['destination-queues'])
-                if ('group' in bot_config and
-                        bot_config['group'] in ['Parser', 'Expert', 'Output']):
-                    if ('source-queue' not in files[PIPELINE_CONF_FILE][bot_id] or
-                            not isinstance(files[PIPELINE_CONF_FILE][bot_id]['source-queue'], str)):
-                        check_logger.error('Misconfiguration: No source queue for %r.', bot_id)
-                        retval = 1
-                    else:
-                        all_queues.add(files[PIPELINE_CONF_FILE][bot_id]['source-queue'])
-                        all_queues.add(files[PIPELINE_CONF_FILE][bot_id]['source-queue'] + '-internal')
+            if ('group' in bot_config and bot_config['group'] in ['Collector', 'Parser', 'Expert']):
+                if ('parameters' not in bot_config or 'destination_queues' not in bot_config['parameters'] or
+                   (isinstance(bot_config['parameters']['destination_queues'], list) and len(bot_config['parameters']['destination_queues']) < 1) or
+                   (isinstance(bot_config['parameters']['destination_queues'], dict) and '_default' not in bot_config['parameters']['destination_queues'])):
+                    check_logger.error('Misconfiguration: No (default) destination queue for %r.', bot_id)
+                    retval = 1
+                else:
+                    all_queues = all_queues.union(bot_config['parameters']['destination_queues'])
+            if ('group' in bot_config and bot_config['group'] in ['Parser', 'Expert', 'Output']):
+                if ('parameters' in bot_config and 'source_queue' in bot_config['parameters'] and isinstance(bot_config['parameters']['source_queue'], str)):
+                    all_queues.add(bot_config['parameters']['source_queue'])
+                    all_queues.add(f"{bot_config['parameters']['source_queue']}-internal")
+                else:
+                    all_queues.add(f"{bot_id}-queue")
+                    all_queues.add(f"{bot_id}-queue-internal")
         # ignore allowed orphaned queues
         allowed_orphan_queues = set(getattr(self.parameters, 'intelmqctl_check_orphaned_queues_ignore', ()))
         if not no_connections:
             try:
-                pipeline = PipelineFactory.create(self.parameters, logger=self.logger)
+                pipeline = PipelineFactory.create(logger=self.logger, pipeline_args=self.parameters.__dict__)
                 pipeline.set_queues(None, "source")
                 pipeline.connect()
                 orphan_queues = "', '".join(pipeline.nonempty_queues() - all_queues - allowed_orphan_queues)
@@ -1541,21 +1540,22 @@ Get some debugging output on the settings and the environment (to be extended):
 
         check_logger.info('Checking for bots.')
         for bot_id, bot_config in files[RUNTIME_CONF_FILE].items():
-            # importable module
-            try:
-                bot_module = importlib.import_module(bot_config['module'])
-            except ImportError as exc:
-                check_logger.error('Incomplete installation: Bot %r not importable: %r.', bot_id, exc)
-                retval = 1
-                continue
-            bot = getattr(bot_module, 'BOT')
-            bot_parameters = files[DEFAULTS_CONF_FILE].copy()
-            bot_parameters.update(bot_config.get('parameters', {}))  # the parameters field may not exist
-            bot_check = bot.check(bot_parameters)
-            if bot_check:
-                for log_line in bot_check:
-                    getattr(check_logger, log_line[0])("Bot %r: %s" % (bot_id, log_line[1]))
-        for group in files[BOTS_FILE].values():
+            if bot_id != 'global':
+                # importable module
+                try:
+                    bot_module = importlib.import_module(bot_config['module'])
+                except ImportError as exc:
+                    check_logger.error('Incomplete installation: Bot %r not importable: %r.', bot_id, exc)
+                    retval = 1
+                    continue
+                bot = getattr(bot_module, 'BOT')
+                bot_parameters = utils.get_global_settings()
+                bot_parameters.update(bot_config.get('parameters', {}))  # the parameters field may not exist
+                bot_check = bot.check(bot_parameters)
+                if bot_check:
+                    for log_line in bot_check:
+                        getattr(check_logger, log_line[0])("Bot %r: %s" % (bot_id, log_line[1]))
+        for group in utils.list_all_bots().values():
             for bot_id, bot in group.items():
                 if subprocess.call(['which', bot['module']], stdout=subprocess.DEVNULL,
                                    stderr=subprocess.DEVNULL):
@@ -1642,14 +1642,20 @@ Get some debugging output on the settings and the environment (to be extended):
                                  state_file)
                 return 0, 'success'
             try:
-                utils.write_configuration(state_file, state, new=True)
+                utils.write_configuration(state_file, state, new=True, useyaml=False)
             except Exception as exc:
                 self.logger.error('Error writing state file %r: %s.', state_file, exc)
                 return 1, 'Error writing state file %r: %s.' % (state_file, exc)
             self.logger.info('Successfully wrote initial state file.')
 
-        defaults = utils.load_configuration(DEFAULTS_CONF_FILE)
         runtime = utils.load_configuration(RUNTIME_CONF_FILE)
+        try:
+            # remove global defaults, handled by 'defaults'
+            del runtime['global']
+        except KeyError:
+            # no global parameters is ok
+            pass
+        defaults = utils.get_global_settings()
         harmonization = utils.load_configuration(HARMONIZATION_CONF_FILE)
         if dry_run:
             self.logger.info('Doing a dry run, not writing anything now.')
@@ -1673,12 +1679,11 @@ Get some debugging output on the settings and the environment (to be extended):
                     upgrades, function)(defaults, runtime, harmonization, dry_run)
                 # Handle changed configurations
                 if retval is True and not dry_run:
-                    utils.write_configuration(DEFAULTS_CONF_FILE, defaults_new,
-                                              backup=not no_backup)
+                    runtime_new['global'] = defaults_new
                     utils.write_configuration(RUNTIME_CONF_FILE, runtime_new,
                                               backup=not no_backup)
                     utils.write_configuration(HARMONIZATION_CONF_FILE, harmonization_new,
-                                              backup=not no_backup)
+                                              backup=not no_backup, useyaml=False)
             except Exception:
                 self.logger.exception('Upgrade %r failed, please report this bug '
                                       'with the shown traceback.',
@@ -1709,7 +1714,7 @@ Get some debugging output on the settings and the environment (to be extended):
             state['upgrades'][function] = result['success']
             if not dry_run:
                 utils.write_configuration(state_file, state,
-                                          backup=not no_backup)
+                                          backup=not no_backup, useyaml=False)
 
             if result['success']:
                 return 0, 'success'
@@ -1825,7 +1830,7 @@ Get some debugging output on the settings and the environment (to be extended):
                 # some upgrade function had a problem
                 if not dry_run:
                     utils.write_configuration(state_file, state,
-                                              backup=not no_backup)
+                                              backup=not no_backup, useyaml=False)
                 self.logger.error('Some migration did not succeed or '
                                   'manual intervention is needed. Look at '
                                   'the output above. Afterwards, re-run '
@@ -1833,14 +1838,13 @@ Get some debugging output on the settings and the environment (to be extended):
 
             try:
                 if not dry_run:
-                    utils.write_configuration(DEFAULTS_CONF_FILE, defaults,
-                                              backup=not no_backup)
+                    runtime['global'] = defaults
                     utils.write_configuration(RUNTIME_CONF_FILE, runtime,
                                               backup=not no_backup)
                     utils.write_configuration(HARMONIZATION_CONF_FILE, harmonization,
-                                              backup=not no_backup)
+                                              backup=not no_backup, useyaml=False)
             except Exception as exc:
-                self.logger.error('Writing defaults or runtime configuration '
+                self.logger.error('Writing runtime configuration '
                                   'did not succeed: %s\nFix the problem and '
                                   'afterwards, re-run this program.',
                                   exc)
@@ -1854,7 +1858,7 @@ Get some debugging output on the settings and the environment (to be extended):
 
             if not dry_run:
                 utils.write_configuration(state_file, state,
-                                          backup=not no_backup)
+                                          backup=not no_backup, useyaml=False)
 
         if error:
             return 1, 'error'
@@ -1874,8 +1878,7 @@ Get some debugging output on the settings and the environment (to be extended):
             variables = globals()
             if RETURN_TYPE == 'text':
                 print('Paths:')
-            for path in ('BOTS_FILE', 'DEFAULTS_CONF_FILE',
-                         'HARMONIZATION_CONF_FILE', 'PIPELINE_CONF_FILE',
+            for path in ('HARMONIZATION_CONF_FILE',
                          'RUNTIME_CONF_FILE', 'VAR_RUN_PATH', 'STATE_FILE_PATH',
                          'DEFAULT_LOGGING_PATH', '__file__',
                          'CONFIG_DIR', 'ROOT_DIR'):
