@@ -1,3 +1,7 @@
+# SPDX-FileCopyrightText: 2016 Sebastian Wagner
+#
+# SPDX-License-Identifier: AGPL-3.0-or-later
+
 # -*- coding: utf-8 -*-
 """
 Testing Mail Attach collector
@@ -18,6 +22,7 @@ REPORT_FOOBARZIP = {
                     'extra.email_from': 'wagner@cert.at',
                     'extra.email_message_id': '<07ce0153-060b-f48d-73d9-d92a20b3b3aa@cert.at>',
                     'extra.email_subject': 'foobar zip',
+                    'extra.email_date': 'Tue, 3 Sep 2019 16:57:40 +0200',
                     'feed.accuracy': 100.0,
                     'feed.name': 'IMAP Feed',
                     'raw': base64_encode('bar text\n'),
