@@ -1,3 +1,7 @@
+# SPDX-FileCopyrightText: 2019 Sebastian Wagner
+#
+# SPDX-License-Identifier: AGPL-3.0-or-later
+
 """
 Tests the upgrade functions.
 """
@@ -463,6 +467,28 @@ V230_MALWAREDOMAINLIST_IN = {
         }
     }
 }
+V233_FEODOTRACKER_BROWSE_IN = {
+'Feodo-tracker-browse-parser': {
+    'module': "intelmq.bots.parsers.html_table.parser",
+    'parameters': {
+        'columns': 'time.source,source.ip,malware.name,status,extra.SBL,source.as_name,source.geolocation.cc'.split(','),
+        'type': 'c2server',
+        'ignore_values': ',,,,Not listed,,',
+        'skip_table_head': True,
+    }
+}
+}
+V233_FEODOTRACKER_BROWSE_OUT = {
+'Feodo-tracker-browse-parser': {
+    'module': "intelmq.bots.parsers.html_table.parser",
+    'parameters': {
+        'columns': 'time.source,source.ip,malware.name,status,source.as_name,source.geolocation.cc',
+        'type': 'c2server',
+        'ignore_values': ',,,,,',
+        'skip_table_head': True,
+    }
+}
+}
 
 def generate_function(function):
     def test_function(self):
@@ -511,8 +537,8 @@ class TestUpgradeLib(unittest.TestCase):
         self.assertTrue(result[0])
         self.assertEqual(V210_EXP, result[2])
 
-    def test_v211_harmonization(self):
-        """ Test v211_harmonization """
+    def test_harmonization(self):
+        """ Test harmonization: Addition of extra to report """
         result = upgrades.harmonization({}, {}, V210_HARM, False)
         self.assertTrue(result[0])
         self.assertEqual(HARM, result[3])
@@ -645,6 +671,12 @@ class TestUpgradeLib(unittest.TestCase):
                          result[0])
         self.assertEqual(V230_MALWAREDOMAINLIST_IN, result[2])
 
+    def test_v233_feodotracker_browse(self):
+        """ Test v233_feodotracker_browse """
+        result = upgrades.v233_feodotracker_browse({}, V233_FEODOTRACKER_BROWSE_IN, {}, False)
+        self.assertTrue(result[0])
+        self.assertEqual(V233_FEODOTRACKER_BROWSE_OUT, result[2])
+
 
 for name in upgrades.__all__:
     setattr(TestUpgradeLib, 'test_function_%s' % name,
